package GyroscopicPrecession "Package for the GyroscopicPrecession example"
  package Components "Components used in the GyroscopicPrecession example."
    extends Modelica.Icons.Package;

    model RotorLiftForce "Generator for periodic impulses"
      Modelica.Blocks.Sources.Constant const(k = 0) annotation(Placement(visible = true, transformation(origin = {-70, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Sources.Pulse pulse(period = 2 * 3.1415927 / angularVelocity, width = duty, amplitude = magnitude, startTime = phase / angularVelocity + liftStart, offset = magnitudeOffset, nperiod = 8) annotation(Placement(visible = true, transformation(origin = {-70, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Sources.Constant const1(k = 0) annotation(Placement(visible = true, transformation(origin = {-70, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.MultiBody.Forces.WorldForce force(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_b, color = {0, 0, 200}) annotation(Placement(visible = true, transformation(origin = {10, 0}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
      parameter Modelica.Units.SI.AngularVelocity angularVelocity "Velocity of the rotor blade";
      parameter Modelica.Units.SI.Angle phase "Phase ofset of the rotor blade";
      parameter Modelica.Units.SI.Period duty "Force duty";
      parameter Modelica.Units.SI.Force magnitude "Force magnitude";
      Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b annotation(Placement(visible = true, transformation(origin = {60, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0), iconTransformation(origin = {100, 0}, extent = {{-16, -16}, {16, 16}}, rotation = 0)));
      parameter Modelica.Units.SI.Force magnitudeOffset "Force offset";
      parameter Modelica.Units.SI.Force liftStart "Start force";
    equation
      connect(force.frame_b, frame_b) annotation(Line(visible = true, origin = {40, 0}, points = {{-20, 0}, {20, 0}}, color = {95, 95, 95}));
      connect(const1.y, force.force[3]) annotation(Line(visible = true, origin = {-32.75, -25}, points = {{-26.25, -25}, {12.75, -25}, {12.75, 25}, {30.75, 25}}, color = {1, 37, 163}));
      connect(pulse.y, force.force[2]) annotation(Line(visible = true, origin = {-30.5, 0}, points = {{-28.5, 0}, {28.5, 0}}, color = {1, 37, 163}));
      connect(const.y, force.force[1]) annotation(Line(visible = true, origin = {-32.75, 25}, points = {{-26.25, 25}, {12.75, 25}, {12.75, -25}, {30.75, -25}}, color = {1, 37, 163}));
      annotation(Documentation(info = "<html><div>
<div>This component produces a series of periodic impulses of force in the positive <strong>y</strong> direction. It is used in the helicopter model as a simplified description of how a helicopter controls its rotor blades to give lift force at a proper angle during every revolution in order to accelerate forward.</div>
</div></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}, radius = 25), Text(visible = true, origin = {-0, -2.329}, extent = {{-68.384, -22.328}, {68.384, 22.329}}, textString = "Rotor Lift")}), Diagram(coordinateSystem(extent = {{-100, -80}, {80, 80}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
    end RotorLiftForce;
    annotation(Documentation(info = "<html></html>"), Diagram(coordinateSystem(extent = {{-148.5, -105.0}, {148.5, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end Components;

  model HelicopterModel "Example of how gyroscopic precession works on a simple helicopter model."
    extends Modelica.Icons.Example;
    inner Modelica.Mechanics.MultiBody.World world(g = 3, animateWorld = false, animateGravity = false) "World component." annotation(Placement(visible = true, transformation(origin = {-90, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.BodyBox helicopterBody(r = {0, 0.2, 0}, r_shape = {0, -0.2, 0}, width = 0.4, height = 0.4, density = 500, r_0.start = {0, -0.2, 0}, color = {155, 0, 0}, r_0.fixed = true, v_0.fixed = true, angles_fixed = true, w_0_fixed = true) "Helicopter body part." annotation(Placement(visible = true, transformation(origin = {-40, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder rotorAxis(r = {0, 0.2, 0}, diameter = 0.05, density = 500, color = {155, 155, 155}, specularCoefficient = 1) "Axis of the rotor." annotation(Placement(visible = true, transformation(origin = {-10, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder rotor(r = {0, 0.05, 0}, diameter = 0.8, density = 500, color = {155, 155, 155}, specularCoefficient = 1) "Rotor of the helicopter." annotation(Placement(visible = true, transformation(origin = {60, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Joints.Revolute revolute(n = {0, 1, 0}, a.start = 0, w.start = rotorSpeed, w.fixed = true) "Joint to allow rotation of the rotor." annotation(Placement(visible = true, transformation(origin = {20, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    parameter Modelica.Units.SI.AngularVelocity rotorSpeed = 100 "Speed of the rotor";
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation blade4(r = {0, 0, -0.4}) "Rotor blade 4." annotation(Placement(visible = true, transformation(origin = {60, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation blade2(r = {0, 0, 0.4}) "Rotor blade 2." annotation(Placement(visible = true, transformation(origin = {60, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation blade3(r = {-0.4, 0, 0}) "Rotor blade 3." annotation(Placement(visible = true, transformation(origin = {60, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation blade1(r = {0.4, 0, 0}) "Rotor blade 1." annotation(Placement(visible = true, transformation(origin = {60, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    parameter Modelica.Units.SI.Force liftForce = 500 "Lift force per blade";
    Modelica.Mechanics.MultiBody.Parts.BodyBox helicopterTail(r = {1, 0, 0}, density = 0, color = {155, 0, 0}) "Visualizer for the helicopter tail." annotation(Placement(visible = true, transformation(origin = {-70, 50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Visualizers.FixedShape ground(widthDirection = {0, 0, 1}, length = 40, width = 40, height = 0.002, color = {156, 170, 155}, r_shape = {-20, -0.5, 0}) "Visualizer for the ground." annotation(Placement(visible = true, transformation(origin = {-30, -50}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Visualizers.FixedShape landingPlatform(widthDirection = {0, 0, 1}, length = 5, width = 5, height = 0.004, color = {74, 74, 74}, r_shape = {-2.5, -0.5, 0}) "visualizer for the helicopter platform." annotation(Placement(visible = true, transformation(origin = {-30, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    GyroscopicPrecession.Components.RotorLiftForce rotorLiftForce1(angularVelocity = rotorSpeed, phase = 0, duty = 10, magnitude = liftForce, magnitudeOffset = 28.0, liftStart = 0.012) "Lift force applied to rotor blade 1." annotation(Placement(visible = true, transformation(origin = {90, 30}, extent = {{-10, -10}, {10, 10}}, rotation = -180)));
    GyroscopicPrecession.Components.RotorLiftForce rotorLiftForce3(angularVelocity = rotorSpeed, phase = 3.1415927, duty = 10, magnitude = liftForce, magnitudeOffset = 28.0, liftStart = 0.012) "Lift force applied to rotor blade 3." annotation(Placement(visible = true, transformation(origin = {90, 5}, extent = {{-10, -10}, {10, 10}}, rotation = -180)));
    GyroscopicPrecession.Components.RotorLiftForce rotorLiftForce2(angularVelocity = rotorSpeed, phase = 3.1415927 / 2, duty = 10, magnitude = liftForce, magnitudeOffset = 28.0, liftStart = 0.012) "Lift force applied to rotor blade 2." annotation(Placement(visible = true, transformation(origin = {90, -25}, extent = {{-10, -10}, {10, 10}}, rotation = -180)));
    GyroscopicPrecession.Components.RotorLiftForce rotorLiftForce4(angularVelocity = rotorSpeed, phase = 3 * 3.1415927 / 2, duty = 10, magnitude = liftForce, magnitudeOffset = 28.0, liftStart = 0.012) "Lift force applied to rotor blade 4." annotation(Placement(visible = true, transformation(origin = {90, -50}, extent = {{-10, -10}, {10, 10}}, rotation = -180)));
  equation
    connect(rotorLiftForce4.frame_b, blade4.frame_b) annotation(Line(visible = true, origin = {74.056, -45}, points = {{5.944, -5}, {-0.944, -5}, {-0.944, 5}, {-4.056, 5}}, color = {95, 95, 95}));
    connect(rotorLiftForce2.frame_b, blade2.frame_b) annotation(Line(visible = true, origin = {74.056, -22.5}, points = {{5.944, -2.5}, {-0.944, -2.5}, {-0.944, 2.5}, {-4.056, 2.5}}, color = {95, 95, 95}));
    connect(rotorLiftForce3.frame_b, blade3.frame_b) annotation(Line(visible = true, origin = {74.056, 2.5}, points = {{5.944, 2.5}, {-0.944, 2.5}, {-0.944, -2.5}, {-4.056, -2.5}}, color = {95, 95, 95}));
    connect(rotorLiftForce1.frame_b, blade1.frame_b) annotation(Line(visible = true, origin = {74.056, 25}, points = {{5.944, 5}, {-0.944, 5}, {-0.944, -5}, {-4.056, -5}}, color = {95, 95, 95}));
    connect(landingPlatform.frame_a, world.frame_b) annotation(Line(visible = true, origin = {-60, -40}, points = {{20, 10}, {0, 10}, {0, -10}, {-20, -10}}, color = {95, 95, 95}));
    connect(ground.frame_a, world.frame_b) annotation(Line(visible = true, origin = {-60, -50}, points = {{20, 0}, {-20, 0}}, color = {95, 95, 95}));
    connect(helicopterTail.frame_b, helicopterBody.frame_a) annotation(Line(visible = true, origin = {-55, 50}, points = {{-5, 0}, {5, 0}}, color = {95, 95, 95}));
    connect(blade1.frame_a, rotor.frame_a) annotation(Line(visible = true, origin = {45, 35}, points = {{5, -15}, {-5, -15}, {-5, 15}, {5, 15}}, color = {95, 95, 95}));
    connect(blade3.frame_a, blade1.frame_a) annotation(Line(visible = true, origin = {45, 10}, points = {{5, -10}, {-5, -10}, {-5, 10}, {5, 10}}, color = {95, 95, 95}));
    connect(blade2.frame_a, blade3.frame_a) annotation(Line(visible = true, origin = {45, -10}, points = {{5, -10}, {-5, -10}, {-5, 10}, {5, 10}}, color = {95, 95, 95}));
    connect(blade4.frame_a, blade2.frame_a) annotation(Line(visible = true, origin = {45, -30}, points = {{5, -10}, {-5, -10}, {-5, 10}, {5, 10}}, color = {95, 95, 95}));
    connect(rotor.frame_a, revolute.frame_b) annotation(Line(visible = true, origin = {67.244, 40}, points = {{-17.244, 10}, {-7.244, 10}, {-7.244, 10}, {-37.244, 10}}, color = {95, 95, 95}));
    connect(helicopterBody.frame_b, rotorAxis.frame_a) annotation(Line(visible = true, origin = {-25, 50}, points = {{-5, 0}, {5, 0}}, color = {95, 95, 95}));
    connect(rotorAxis.frame_b, revolute.frame_a) annotation(Line(visible = true, origin = {14.006, 40}, points = {{-14.006, 10}, {5.994, 10}, {5.994, 10}, {-4.006, 10}}, color = {95, 95, 95}));
    annotation(__Wolfram(ControlPanels(Panel(identifier = "gracious-perelman", title = "Model parameters", elements = {Slider(variable = liftForce, label = "Lift force per blade", min = 100, max = 700), Slider(variable = rotorSpeed, label = "Speed of the rotor", min = 10, max = 200)})), Animation(cameras = {Camera(name = "Default", distance = 11.98467716201129, rotation = {0, 0, 0, 1}, follow = helicopterBody.frameTranslation.shape, followMode = "NODE_CENTER")})), Documentation(info = "<html>
<p>This model simulates how gyroscopic precession is used in a helicopter to make it move forward.</p>

<h4>Helicopter Model</h4>

<p>To get a helicopter to move forward is a bit more complex than it might seem. This is actually done by using gyroscopic precession. The principle is not more complicated than a plain gyro, but in reality, as usual, it takes a lot of effort to put the theories into practice. On a real helicopter, precession is achieved by controlling the blades of the rotor so that they produce extra force upward on the left side (provided the rotor is spinning counterclockwise), which introduces a clockwise torque on the helicopter body as seen from behind. This in turn generates a precession tilting the vehicle nose downward.</p>

<p>As decribed in the description of the gyro model in this example, you can think of this phenomenon as a 90 degree delay of the lift force; a force pushing upward on the left side manifests 90 degrees later, creating a lifting force on the rear side of the rotor that after tilting the nose of the helicopter downward propulses it forward.</p>

<p>In this example, the rotor is modeled as a solid circular disk with four forces acting on four evenly spaced points on the edge of the disk. The four forces act as periodic impulses on the \"rotor blades\", simulating the lift force coming from the change in pitch angle of the rotor blades on a helicopter.</p>

<p style=\"text-align: center;\"><img src=\"modelica://GyroscopicPrecession/DocumentationFiles/GyroCopter.png\" style=\"width:600px\" alt=\"\" /></p>

<h4>Simulation</h4>

<p>By simulating the model, you can try different values of the parameters that are relevant for the precession. If you increase the amplitude of the lift force, for example, you will see that the helicopter rises up in the air faster, but also tilts forward more aggressively.</p>

<p>To simulate the model and see the generated 3D animation, follow the steps below:</p>

<p>Click the Simulate button: <img src=\"modelica://GyroscopicPrecession/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"\" /></p>

<p>Click the Animation button: <img src=\"modelica://GyroscopicPrecession/DocumentationFiles/animate.svg\" style=\"width:20px\" alt=\"animate\" /></p>
<p>Use your mouse or trackpad to drag the animation to a good angle and zoom in with your scroll wheel or by using the trackpad. Then click the Play button: <img src=\"modelica://GyroscopicPrecession/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"play\" /></p>

<p>To see exactly what the velocities or forces are, you can plot them by opening the Plot tab. Double click the PositionAndLifForce plot in the Stored Plots section to see a plot of the position of the helicopter and the lift forces affection the four blades.</p>

<h4>Discussion</h4>

<p>After simulating, you may notice that the helicopter does not move particularly smoothly. If you try to apply an external angular momentum directly to the body, you will see that the ride seems a lot smoother. The oscillating behavior of the vehicle is due to the fact that the momentum created by the periodic forces on the \"rotor blades\" are not continuous, but based on a fairly crude model. In a real helicopter, these kinds of problems are carefully controlled and compensated for. You also need to compensate for many other different factors that are not important for this example.</p>
</html>", figures = {Figure(title = "Position and lift force", identifier = "07200", preferred = true, plots = {Plot(curves = {Curve(y = rotor.r_0[2], legend = "Helicopter Height"), Curve(y = rotor.r_0[1], legend = "X-position")}), Plot(curves = {Curve(y = rotorLiftForce1.pulse.y, legend = "Lift force blade 1"), Curve(y = rotorLiftForce2.pulse.y, legend = "Lift force blade 2"), Curve(y = rotorLiftForce3.pulse.y, legend = "Lift force blade 3"), Curve(y = rotorLiftForce4.pulse.y, legend = "Lift force blade 4")})}), Figure(title = "XY position", identifier = "1d592", plots = {Plot(curves = {Curve(x = helicopterBody.r_0[1], y = helicopterBody.r_0[2], legend = "Helicopter X-Y position")})})}), experiment(StartTime = 0.0, StopTime = 3.5, NumberOfIntervals = 2000, __Wolfram_Algorithm = "cvodes", Tolerance = 1e-6, __Wolfram_MaxStepSize = 0, __Wolfram_EventHysteresisEpsilon = 1e-10, __Wolfram_AccurateEventDetection = true, __Wolfram_NonlinearSolverTolerance = 1e-12, __Wolfram_MaxInternalStepCount = 0, __Wolfram_CheckMinMax = false, __Wolfram_StopAtSteadyState = false, __Wolfram_SynchronizeWithRealTime = false), Diagram(coordinateSystem(extent = {{-120, -80}, {120, 70}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end HelicopterModel;

  model GyroModel "Example of how gyroscopic precession works on a simple gyro."
    extends Modelica.Icons.Example;
    inner Modelica.Mechanics.MultiBody.World world(animateWorld = false, animateGravity = false) "world component" annotation(Placement(visible = true, transformation(origin = {-130, -80}, extent = {{-10, -10}, {10, 10}}, rotation = -720)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder mount(r = {0, 0.002, 0}, color = {155, 155, 155}, diameter = 0.08, specularCoefficient = 1) "mount of the gyro" annotation(Placement(visible = true, transformation(origin = {-70, -80}, extent = {{-10, -10}, {10, 10}}, rotation = -720)));
    parameter Modelica.Units.SI.AngularVelocity spin = 500 "Angular velocity of inner rotor";
    parameter Modelica.Units.SI.Force forceMagnitude = 0.3 "Magnitude of applied force";
    Modelica.Mechanics.MultiBody.Visualizers.VoluminousWheel outerRing(final rRim = 0.041, width = 0.005, color = {155, 155, 155}, specularCoefficient = 1, final rTire = 0.045, final rCurvature = 0.040) "outer ring of the gyro" annotation(Placement(visible = true, transformation(origin = {100.85, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation(r = {0, 0.041, 0}, animation = false) "distance from the outer ring to the center of the gyro" annotation(Placement(visible = true, transformation(origin = {0, -40}, extent = {{-10, -10}, {10, 10}}, rotation = -270)));
    Modelica.Mechanics.MultiBody.Parts.FixedRotation outerRinFixedgRotation(animation = false, angle = 90) "fixed rotation to line up the outer ring initially" annotation(Placement(visible = true, transformation(origin = {70, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Visualizers.VoluminousWheel innerRing(final rRim = 0.036, final rTire = 0.040, width = 0.005, color = {155, 155, 155}, specularCoefficient = 1, final rCurvature = 0.035) "inner ring of the gyro" annotation(Placement(visible = true, transformation(origin = {100.85, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedRotation innerRingFixedRotation(animation = false, angle = 90) "fixed rotation to line up the inner ring initially" annotation(Placement(visible = true, transformation(origin = {69.15, 30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r = {0, 0.005, 0}, animation = true, width = 0.003, color = {155, 155, 155}, specularCoefficient = 1) "distance from the mount to the outer ring" annotation(Placement(visible = true, transformation(origin = {0, -65}, extent = {{-10, -10}, {10, 10}}, rotation = -270)));
    Modelica.Mechanics.MultiBody.Parts.BodyCylinder rotor(diameter = 0.07, r = {0.0025, 0, 0}, r_shape = {-0.0025, 0, 0}, color = {100, 100, 100}, specularCoefficient = 1) "the rotor of the gyro" annotation(Placement(visible = true, transformation(origin = {100, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedRotation rotorFixedRotation(animation = false, angle = 90, n = {0, 1, 0}) "fixed rotation to line up the rotor initially" annotation(Placement(visible = true, transformation(origin = {70, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Joints.Revolute revoluteX(n = {1, 0, 0}, animation = false) "revolution around the x-axis" annotation(Placement(visible = true, transformation(origin = {0, 15}, extent = {{-10, -10}, {10, 10}}, rotation = -270)));
    Modelica.Mechanics.MultiBody.Joints.Revolute revoluteZ(n = {0, 0, 1}, animation = false, w.start = spin) "revolution around the Z-axis" annotation(Placement(visible = true, transformation(origin = {0, 45}, extent = {{-10, -10}, {10, 10}}, rotation = -270)));
    Modelica.Mechanics.MultiBody.Joints.Revolute revoluteY(n = {0, 1, 0}, animation = false) "revolution around the Y-axis" annotation(Placement(visible = true, transformation(origin = {0, -15}, extent = {{-10, -10}, {10, 10}}, rotation = -270)));
    Modelica.Mechanics.MultiBody.Visualizers.FixedShape rotationIndicator(color = {255, 255, 0}, length = 0.002, width = 0.04, height = 0.002, specularCoefficient = 1) "indicator of the orientation of the rotor" annotation(Placement(visible = true, transformation(origin = {130, 85}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Parts.FixedTranslation rotationIndicatorOrientator(r = {-0.003, 0, 0}, animation = false) "placement for the indicator of the rotor orientation" annotation(Placement(visible = true, transformation(origin = {100, 85}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
    Modelica.Mechanics.MultiBody.Forces.WorldForce force(resolveInFrame = Modelica.Mechanics.MultiBody.Types.ResolveInFrameB.frame_b) "external force" annotation(Placement(visible = true, transformation(origin = {70, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Blocks.Sources.Pulse pulse[3](amplitude = {0, 0, -forceMagnitude}, period = 100, width = 3, startTime = 1, nperiod = 1) "signal to external force" annotation(Placement(visible = true, transformation(origin = {30, -80}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Mechanics.MultiBody.Visualizers.FixedShape baseOrientationIndicator(color = {0, 0, 200}, length = 0.002, width = 0.0012, height = 0.06, specularCoefficient = 1) "indicator of the orientation of the mount" annotation(Placement(visible = true, transformation(origin = {-70, -50}, extent = {{-10, -10}, {10, 10}}, rotation = -180)));
    parameter Modelica.Units.NonSI.Angle_deg degreeOfForce = 45 "Angle of applied force";
    Modelica.Mechanics.MultiBody.Parts.FixedRotation forceOrientator(animation = false, angle = 90, r = {0.038 * cos(Modelica.Units.Conversions.from_deg(degreeOfForce)), 0.038 * sin(Modelica.Units.Conversions.from_deg(degreeOfForce)), 0}, n = {0, 0, 1}) "displacement showing where to put the external force" annotation(Placement(visible = true, transformation(origin = {70, -25}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  equation
    connect(mount.frame_a, world.frame_b) annotation(Line(visible = true, origin = {-100, -80}, points = {{20, 0}, {-20, 0}}, color = {95, 95, 95}));
    connect(outerRinFixedgRotation.frame_b, outerRing.frame_a) annotation(Line(visible = true, origin = {85.425, 0}, points = {{-5.425, 0}, {5.425, 0}}, color = {95, 95, 95}));
    connect(innerRingFixedRotation.frame_b, innerRing.frame_a) annotation(Line(visible = true, origin = {85, 30}, points = {{-5.85, 0}, {5.85, 0}}, color = {95, 95, 95}));
    connect(fixedTranslation1.frame_b, fixedTranslation.frame_a) annotation(Line(visible = true, origin = {0, -52.5}, points = {{0, -2.5}, {0, 2.5}}, color = {95, 95, 95}));
    connect(fixedTranslation1.frame_a, mount.frame_b) annotation(Line(visible = true, origin = {-20, -78.333}, points = {{20, 3.333}, {20, -1.667}, {-40, -1.667}}, color = {95, 95, 95}));
    connect(rotor.frame_a, rotorFixedRotation.frame_b) annotation(Line(visible = true, origin = {85, 60}, points = {{5, 0}, {-5, 0}}, color = {95, 95, 95}));
    connect(revoluteX.frame_b, innerRingFixedRotation.frame_a) annotation(Line(visible = true, origin = {19.717, 28.333}, points = {{-19.717, -3.333}, {-19.717, 1.667}, {39.433, 1.667}}, color = {95, 95, 95}));
    connect(revoluteX.frame_b, revoluteZ.frame_a) annotation(Line(visible = true, origin = {0, 30}, points = {{0, -5}, {0, 5}}, color = {95, 95, 95}));
    connect(revoluteZ.frame_b, rotorFixedRotation.frame_a) annotation(Line(visible = true, origin = {20, 58.333}, points = {{-20, -3.333}, {-20, 1.667}, {40, 1.667}}, color = {95, 95, 95}));
    connect(fixedTranslation.frame_b, revoluteY.frame_a) annotation(Line(visible = true, origin = {0, -27.5}, points = {{0, -2.5}, {0, 2.5}}, color = {95, 95, 95}));
    connect(revoluteY.frame_b, revoluteX.frame_a) annotation(Line(visible = true, points = {{0, -5}, {0, 5}}, color = {95, 95, 95}));
    connect(revoluteY.frame_b, outerRinFixedgRotation.frame_a) annotation(Line(visible = true, origin = {20, -1.667}, points = {{-20, -3.333}, {-20, 1.667}, {40, 1.667}}, color = {95, 95, 95}));
    connect(rotationIndicatorOrientator.frame_a, rotor.frame_a) annotation(Line(visible = true, origin = {85.506, 72.5}, points = {{4.494, 12.5}, {-0.506, 12.5}, {-0.506, -12.5}, {4.494, -12.5}}, color = {95, 95, 95}));
    connect(rotationIndicatorOrientator.frame_b, rotationIndicator.frame_a) annotation(Line(visible = true, origin = {115, 85}, points = {{-5, 0}, {5, 0}}, color = {95, 95, 95}));
    connect(pulse.y, force.force) annotation(Line(visible = true, origin = {49.5, -80}, points = {{-8.5, 0}, {8.5, 0}}, color = {1, 37, 163}));
    connect(baseOrientationIndicator.frame_a, mount.frame_b) annotation(Line(visible = true, origin = {-41.613, -65}, points = {{-18.387, 15}, {1.613, 15}, {1.613, -15}, {-18.387, -15}}, color = {95, 95, 95}));
    connect(forceOrientator.frame_a, innerRingFixedRotation.frame_a) annotation(Line(visible = true, origin = {36.294, 2.5}, points = {{23.706, -27.5}, {-6.294, -27.5}, {-6.294, 27.5}, {22.856, 27.5}}, color = {95, 95, 95}));
    connect(forceOrientator.frame_b, force.frame_b) annotation(Line(visible = true, origin = {93.5, -65}, points = {{-13.5, 40}, {6.5, 40}, {6.5, -15}, {-13.5, -15}}, color = {95, 95, 95}));
    annotation(experiment(StopTime = 10.0, __Wolfram_SynchronizeWithRealTime = false), preferredView = "info", __Wolfram(ControlPanels(Panel(identifier = "sharp-johnson", title = "Model parameters", elements = {Slider(variable = forceMagnitude, label = "Magnitude of applied force", min = 0.01, max = 0.5), Slider(variable = spin, label = "Angular velocity of inner rotor", min = 100, max = 700)}))), Documentation(info = "<html>
<p>This model studies how the torque applied to a rotating object may induce gyroscopic precession.</p>

<h4>Gyro Model</h4>

<p>The gyro in this model consists of a mount on which a rotor is placed in a system of two outer rings. This system makes it possible for the rotor to rotate in all three spatial dimensions (see graphic below). The rotor is given an initial angular velocity, and after a little while an external force acts on the inner ring, inducing a phenomenon called gyroscopic precession. An easy way of visualizing what is going on is to just add a 90 degree delay of the force being applied; a force being applied to the \"3 o'clock\" position acts as if it is being applied at the \"12 o'clock\" position (provided the rotor is spinning counterclockwise).</p>

<p style=\"text-align: center;\"><img src=\"modelica://GyroscopicPrecession.GyroModel/./DocumentationFiles/gyro.png\" style=\"width:300px\" alt=\"\" /></p>

<h4>Simulation</h4>

<p>By simulating the model, you can try different values of the parameters and see how the gyro behaves. One of the most interesting parameters is the degreeOfForce, which will put the force on the inner ring at the degree specified, where 0 degrees is \"3 o'clock\", 90 degrees is \"12 o'clock\", etc. Try different values and check for yourself that the force always gets delayed by 90 degrees. Also try different values of the strength of the force (here called forceMagnitude) and the angular speed (here called spin) of the rotor and see how this affects the motion.</p>

<p>To simulate the model and see the generated 3D animation, follow the steps below:</p>
<p>Click the Simulate button: <img src=\"modelica://GyroscopicPrecession/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"\" /></p>

<p>Click the Animation button: <img src=\"modelica://GyroscopicPrecession/DocumentationFiles/animate.svg\" style=\"width:20px\" alt=\"animate\" /></p>
<p>Use your mouse or trackpad to drag the animation to a good angle and zoom in with your scroll wheel or by using the trackpad. Then click the Play button: <img src=\"modelica://GyroscopicPrecession/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"play\" /></p>


<p>If you would like to examine other things like angles and speeds during the simulation, you can plot these in the Simulation Center. Double click the RevolutionAngles plot in the Stored Plots section to see a plot of the angle of the inner ring.</p>

<h4>Gyroscopic Precession</h4>

<p>Earlier the gyroscopic precession was described as a 90 degree delay of a force applied to a rotating body. While this is perfectly true, there is a more mathematical way of describing it. There are three physical quantities involved in gyroscopic precession: <strong>T</strong>, <strong>&omega;<sub>p</sub></strong>, <strong>&omega;<sub>s</sub></strong>, where <strong>T</strong> is the applied torque, <strong>&omega;<sub>p</sub></strong> is the angular velocity due to precession, and <strong>&omega;<sub>s</sub></strong> is the angular velocity of the rotor. These three make up a right-handed system in this order. This means, for example, that if the rotor spins around the <strong>x</strong> axis and torque is applied around the <strong>y</strong> axis, the gyro starts precessing around the <strong>z</strong> axis. The graphic below illustrates a rotor and the three rotational quantities. The vectors denote the direction of the rotational axes.</p>

<p style=\"text-align: center;\"><img src=\"modelica://GyroscopicPrecession.GyroModel/./DocumentationFiles/GyroPrec.png\" style=\"width:300px\" alt=\"\" /></p>
</html>", figures = {Figure(title = "Revolution angles", identifier = "afaa4", preferred = true, plots = {Plot(curves = {Curve(y = revoluteX.angle, legend = "Revolute X angle"), Curve(y = revoluteY.angle, legend = "Revolute Y angle")})}), Figure(title = "Applied force", identifier = "5557f", plots = {Plot(curves = {Curve(y = force.force[3], legend = "Applied Force")})})}), Diagram(coordinateSystem(extent = {{-160, -120}, {160, 117.912}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end GyroModel;
  annotation(__Wolfram(openClass = GyroscopicPrecession.GyroModel), preferredView = "info", uses(Modelica(version = "4.0.0")), Documentation(info = "<html></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90, -90}, {90, 90}}, radius = 25), Text(visible = true, origin = {12, 20.833}, textColor = {255, 255, 255}, extent = {{-102, -70.833}, {78, 29.167}}, textString = "Ex", fontName = "Arial")}));
end GyroscopicPrecession;
